@@ -10,6 +10,8 @@ from libcst.codemod import CodemodContext, VisitorBasedCodemodCommand
 from libcst.codemod.visitors import AddImportsVisitor
 from libcst.metadata import CodePosition, CodeRange, PositionProvider
 
+from autotyping.guess_type import guess_type_from_argname
+
 _DEFAULT_POSITION = CodePosition(0, 0)
 _DEFAULT_CODE_RANGE = CodeRange(_DEFAULT_POSITION, _DEFAULT_POSITION)
 
@@ -61,12 +63,14 @@ class State:
         default_factory=dict
     )
     only_without_imports: bool = False
+    guess_common_names: bool = False
 
 
 SIMPLE_MAGICS = {
     "__str__": "str",
     "__repr__": "str",
     "__len__": "int",
+    "__length_hint__": "int",
     "__init__": "None",
     "__del__": "None",
     "__bool__": "bool",
@@ -85,38 +89,7 @@ IMPRECISE_MAGICS = {
 }
 
 
-class _SafeAction(argparse.Action):
-    def __call__(
-        self,
-        parser: argparse.ArgumentParser,
-        namespace: argparse.Namespace,
-        values: object,
-        option_string: Optional[str] = ...,
-    ) -> None:
-        namespace.none_return = True
-        namespace.scalar_return = True
-        namespace.annotate_magics = True
-
-
-class _AggressiveAction(_SafeAction):
-    def __call__(
-        self,
-        parser: argparse.ArgumentParser,
-        namespace: argparse.Namespace,
-        values: object,
-        option_string: Optional[str] = ...,
-    ) -> None:
-        super().__call__(parser, namespace, values, option_string)
-        namespace.bool_param = True
-        namespace.int_param = True
-        namespace.float_param = True
-        namespace.str_param = True
-        namespace.bytes_param = True
-        namespace.annotate_imprecise_magics = True
-
-
 class AutotypeCommand(VisitorBasedCodemodCommand):
-
     # Add a description so that future codemodders can see what this does.
     DESCRIPTION: str = "Automatically adds simple type annotations."
     METADATA_DEPENDENCIES = (PositionProvider,)
@@ -166,7 +139,7 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
             "--int-param",
             action="store_true",
             default=False,
-            help=("Automatically add int annotation to parameters with an int default"),
+            help="Automatically add int annotation to parameters with an int default",
         )
         arg_parser.add_argument(
             "--float-param",
@@ -180,7 +153,7 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
             "--str-param",
             action="store_true",
             default=False,
-            help=("Automatically add str annotation to parameters with a str default"),
+            help="Automatically add str annotation to parameters with a str default",
         )
         arg_parser.add_argument(
             "--bytes-param",
@@ -201,10 +174,16 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
             action="store_true",
             default=False,
             help=(
-                "Add annotations to magic methods that are less precise (e.g., Iterable "
-                "for __iter__)"
+                "Add annotations to magic methods that are less precise (e.g., Iterable"
+                " for __iter__)"
             ),
         )
+        arg_parser.add_argument(
+            "--guess-common-names",
+            action="store_true",
+            default=False,
+            help="Guess types from argument name",
+        )
         arg_parser.add_argument(
             "--pyanalyze-report",
             type=str,
@@ -219,15 +198,15 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
         )
         arg_parser.add_argument(
             "--safe",
-            action=_SafeAction,
+            action="store_true",
+            default=False,
             help="Apply all safe transformations",
-            nargs="?",
         )
         arg_parser.add_argument(
             "--aggressive",
-            action=_AggressiveAction,
+            action="store_true",
+            default=False,
             help="Apply all transformations that do not require arguments",
-            nargs="?",
         )
 
     def __init__(
@@ -245,11 +224,24 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
         bytes_param: bool = False,
         float_param: bool = False,
         int_param: bool = False,
+        guess_common_names: bool = False,
         pyanalyze_report: Optional[str] = None,
         only_without_imports: bool = False,
         safe: bool = False,
         aggressive: bool = False,
     ) -> None:
+        if safe or aggressive:
+            none_return = True
+            scalar_return = True
+            annotate_magics = True
+        if aggressive:
+            bool_param = True
+            int_param = True
+            float_param = True
+            str_param = True
+            bytes_param = True
+            annotate_imprecise_magics = True
+            guess_common_names = True
         super().__init__(context)
         param_type_pairs = [
             (bool_param, bool),
@@ -283,12 +275,16 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
                     )
                 ] = metadata
         self.state = State(
-            annotate_optionals=[NamedParam.make(s) for s in annotate_optional]
-            if annotate_optional
-            else [],
-            annotate_named_params=[NamedParam.make(s) for s in annotate_named_param]
-            if annotate_named_param
-            else [],
+            annotate_optionals=(
+                [NamedParam.make(s) for s in annotate_optional]
+                if annotate_optional
+                else []
+            ),
+            annotate_named_params=(
+                [NamedParam.make(s) for s in annotate_named_param]
+                if annotate_named_param
+                else []
+            ),
             none_return=none_return,
             scalar_return=scalar_return,
             param_types={typ for param, typ in param_type_pairs if param},
@@ -296,6 +292,7 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
             annotate_imprecise_magics=annotate_imprecise_magics,
             pyanalyze_suggestions=pyanalyze_suggestions,
             only_without_imports=only_without_imports,
+            guess_common_names=guess_common_names,
         )
 
     def is_stub(self) -> bool:
@@ -504,8 +501,10 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
         if self.state.in_lambda:
             # Lambdas can't have annotations
             return updated_node
+        # don't modify if there's already any annotations set
         if original_node.annotation is not None:
             return updated_node
+        # pyanalyze suggestions
         if self.state.pyanalyze_suggestions and self.context.filename:
             pos = self.get_metadata(
                 PositionProvider, original_node, _DEFAULT_CODE_RANGE
@@ -516,7 +515,7 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
                 suggestion["imports"] and self.state.only_without_imports
             ):
                 for import_line in suggestion["imports"]:
-                    if "." in import_line:
+                    if "." not in import_line:
                         AddImportsVisitor.add_needed_import(self.context, import_line)
                     else:
                         mod, name = import_line.rsplit(".", maxsplit=1)
@@ -526,6 +525,7 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
                 )
                 return updated_node.with_changes(annotation=annotation)
 
+        # infer from default non-None value
         if original_node.default is not None:
             default_type = type_of_expression(original_node.default)
             if default_type is not None and default_type in self.state.param_types:
@@ -535,40 +535,57 @@ class AutotypeCommand(VisitorBasedCodemodCommand):
                     )
                 )
 
+        parameter_name = original_node.name.value
         default_is_none = (
             original_node.default is not None
             and isinstance(original_node.default, libcst.Name)
             and original_node.default.value == "None"
         )
+        # default value is None, i.e. `def foo(bar=None)`
         if default_is_none:
+            # check if user has explicitly specified a type for this name
             for anno_optional in self.state.annotate_optionals:
-                if original_node.name.value == anno_optional.name:
+                if parameter_name == anno_optional.name:
                     return self._annotate_param(
-                        anno_optional, updated_node, optional=True
+                        anno_optional, updated_node, containers=["Optional"]
                     )
+
+        # no default value, i.e. `def foo(bar)`
         elif original_node.default is None:
+            # check if user has explicitly specified a type for this name
             for anno_named_param in self.state.annotate_named_params:
                 if original_node.name.value == anno_named_param.name:
-                    return self._annotate_param(anno_named_param, updated_node)
+                    return self._annotate_param(anno_named_param, updated_node, [])
+
+        # guess type from name
+        if self.state.guess_common_names:
+            guessed_type, containers = guess_type_from_argname(parameter_name)
+            if guessed_type is not None:
+                if default_is_none:
+                    containers += ["Optional"]
+                return self._annotate_param(
+                    NamedParam("", None, guessed_type), updated_node, containers
+                )
+
         return updated_node
 
     def _annotate_param(
-        self, param: NamedParam, updated_node: libcst.Param, optional: bool = False
+        self, param: NamedParam, updated_node: libcst.Param, containers: List[str]
     ) -> libcst.Param:
         if param.module is not None:
             AddImportsVisitor.add_needed_import(
                 self.context, param.module, param.type_name
             )
-        if optional:
-            AddImportsVisitor.add_needed_import(self.context, "typing", "Optional")
-        type_name = libcst.Name(value=param.type_name)
-        if optional:
+        anno = libcst.Name(value=param.type_name)
+
+        for container in containers:
+            # Should be updated when python <3.9 support is dropped
+            AddImportsVisitor.add_needed_import(self.context, "typing", container)
+
             anno = libcst.Subscript(
-                value=libcst.Name(value="Optional"),
-                slice=[libcst.SubscriptElement(slice=libcst.Index(value=type_name))],
+                value=libcst.Name(value=container),
+                slice=[libcst.SubscriptElement(slice=libcst.Index(value=anno))],
             )
-        else:
-            anno = type_name
         return updated_node.with_changes(annotation=libcst.Annotation(annotation=anno))
 
 
